/*
   
 ESUP-portail is a french academic project developed under the GPL (General Public License)
 augmented according to the following :
 
 A binary or source file developped by ESUP-portail can be used or compiled with products under Apache license.
 
 The official english text of the GPL can be found here : http://www.gnu.org/licenses/gpl.html .
 
 A non official french translation can be found here : http://www.linux-France.org/article/these/gpl.html .
 
 The different kinds of licenses governing the products developed by the Apache
 foundation can be found here : http://www.apache.org/licenses .
 
 It follows that you can as well as use download contents as well modify and redistribute
 them provided you respect the GPL terms.
 
 Downloading and using such contents do not provide any guaranty.
 
 Be sure that you have well understood the terms of the license before using the contents it covers.
 
 The ESUP-portail distribution includes the following distributions :
 
 * UPortal :
 software developed by JA-SIG (Java Architecture - Special Interest Group)
 You can find the license page here : http://mis105.udel.edu/ja-sig/uportal/license.html
 
 * CAS :
 SSO solution developed by Yale University
 You can find the project page here : http://www.yale.edu/tp/auth
 
 * Cocoon :
 XML framework distributed by the Apache foundation under Apache license;
 Please find the full text here : http://cocoon.apache.org/2.1/license.html
 
 * Mod_dav:
 A DAV module for Apache web server
 You can find the project page here : http://www.webdav.org/mod_dav
 
 * IMP :
 webmail from Horde application framework
 You can find the project page here : http://www.horde.org
 */
package org.esupportail.portal.channels.CAnnuaire;


import java.util.Iterator;
import java.util.Vector;
import org.jasig.portal.*;
import org.jasig.portal.utils.*;
import org.xml.sax.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class CAnnuaire implements IChannel, ICacheable {
	
	private static final Log log = LogFactory.getLog(CAnnuaire.class);
	
	/****************************/
	/*Variables de programmation*/
	/****************************/
	
	private String annuaireSelected = null;
	
	private final String ETAT_FORMULAIRE = "Formulaire";
	private final String ETAT_TABLEAU_RES = "Tableau";
	private final String ETAT_FICHE_RES = "Fiche";
	private final String ETAT_RECHERCHE = "Recherche";
	private final String ETAT_REFRESH = "Actualisation";
	//attention crit en dur dans le xsl
	private final String ETAT_DETAIL = "Detail";
	private final String ETAT_MAIL = "sendMail";
	////
	
	/****************************/
	/*Variables de configuration*/
	/****************************/
	
	private String etat = null;
	private boolean passeParTab = false;
	
	/*fichier de configuration*/
	private final static String configFilePath =
		"/properties/channels/org_esup/CAnnuaire/CAnnuaire.xml";
	
	/*feuille ssl*/
	private final static String xslLocation = "CAnnuaire.ssl";
	
	/*temp de cache en secondes*/
	private int cacheTimeOut = 500;
	
	/*interface utilisateur*/
	protected userInterface UI;
	
	ChannelStaticData staticData = null;
	ChannelRuntimeData runtimeData = null;
	
	
	/*Servant MailTo*/
	private IServant servant;
	private boolean servantAllowed = false;
	private boolean servantAvailable = false;
	private String nextMail;
	private final static String mailToClass =
		"org.esupportail.portal.channels.CMailTo.CMailToServant";
	private final static String toField ="to";//Mail.TO : pour viter la dpendence on n'utilise pas la constante
	
	
	private config conf;
	
	//Construire le cadre
	public CAnnuaire() {
		conf = config.getInstance(configFilePath);
		UI = new userInterface(conf);
	}
	
	
	
	public ChannelRuntimeProperties getRuntimeProperties() {
		return new ChannelRuntimeProperties();
	}
	
	/** Processes layout-level events coming from the portal
	 * @param ev a portal layout event
	 */
	public void receiveEvent(PortalEvent ev) {
	}
	
	/** Receive static channel data from the portal
	 * @param sd static channel data
	 */
	public void setStaticData(ChannelStaticData sd) {
		this.staticData = sd;
		
		String serveurAuthorise = sd.getParameter("serverView");
		if ((serveurAuthorise == null) || (serveurAuthorise == ""))
			serveurAuthorise = "*";
		UI.setServerView(serveurAuthorise);

		
		if ( (!sd.getPerson().isGuest())
			&& (sd.getParameter("CAnnuaireServantMode")== null)
			){
			servantAllowed=true;
			UI.setMailToAllowed(servantAllowed);
		}
	}
	
	/** Receives channel runtime data from the portal and processes actions
	 * passed to it.  The names of these parameters are entirely up to the channel.
	 * @param rd handle to channel runtime data
	 * @throws PortalException
	 */
	public void setRuntimeData(ChannelRuntimeData rd) throws PortalException {
		this.runtimeData = rd;
		etat = rd.getParameter("etat");
		
		if (etat == null)
			etat = ETAT_FORMULAIRE;
		
		
		if ( (servantAllowed)
			&&(!servantAvailable)
			&&(etat.equals(ETAT_MAIL))
			){
			servantAvailable = true;
			nextMail = new String();
			String id = rd.getParameter("id");
			String attrib = rd.getParameter("attrib");
			Personne p = UI.getResult(id);
			Vector values = (Vector)p.getAttribute(attrib);
			Iterator i = values.iterator();
			//l'envoi de mail ne se fait que sur la premire adresse
			if (i.hasNext()){
				String val = (String)i.next();
				nextMail += val;
			}
		} 
		
		if (servantAvailable){	
			
			String[] att = new String[3];
	        att[0] = nextMail;
	        att[1] = "false";
	        att[2] = "true";
	        runtimeData.setParameterValues(toField,att);
		    
		    
		    try {
                Class servantClass = Class.forName(mailToClass);
                // Instanciation du servant
                Object servantObj = servantClass.newInstance();
                servant = (IServant)servantObj;
            } catch (ClassNotFoundException e) {
                log.error("CAnnuaire::setRuntimeData:ClassNotFoundException : "+e.getMessage());
                servantAvailable = false;
            } catch (InstantiationException e) {
            	log.error("CAnnuaire::setRuntimeData:InstantiationException : "+e.getMessage());
                servantAvailable = false;
            } catch (IllegalAccessException e) {
            	log.error("CAnnuaire::setRuntimeData:IllegalAccessException : "+e.getMessage());
                servantAvailable = false;
            } catch (Exception e) {
            	log.error("CAnnuaire::setRuntimeData:Exception : "+e.getMessage());
                servantAvailable = false;
            }

            //Passage des paramtres
            servant.setStaticData(staticData);
            servant.setRuntimeData(runtimeData);
            
            if (servant.isFinished()) {
                servantAvailable=false;
                etat=ETAT_TABLEAU_RES;
                nextMail = "";
            }		    
		}
		else{

			String btl = runtimeData.getParameter("backToList");
			if ( (btl!=null) && (btl.equals("true"))){
				etat=ETAT_TABLEAU_RES;
			}
	
			annuaireSelected = rd.getParameter("annuaire");
		}
		
	}
	
	/** Output channel content to the portal
	 * @param out a sax document handler
	 */
	public void renderXML(ContentHandler out) throws PortalException {
		
		
		if (servantAvailable) {
			servant.renderXML(out);
			
		}
        else{
        	myRenderXML(out);
        }
	}
		
	private  void myRenderXML(ContentHandler out) throws PortalException {	
		int nbRes = 0;
		
		boolean newQuery=true;
		String uQ = runtimeData.getParameter("updateQuery");
		if (uQ!=null)newQuery=!uQ.matches("true");
		
		String xml = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<RACINE>\n";
		
		if (etat.equals(ETAT_REFRESH)) {
			boolean setOk = UI.setServerWork(annuaireSelected);
			if (!setOk)
				xml += "<ALERT>WARN_SERVER_NOT_FOUND</ALERT>\n";
			etat = ETAT_FORMULAIRE;
		}
		
		
		if (UI.getServerWork() != null) {
			if (etat.equals(ETAT_FORMULAIRE)) {
				xml += UI.getForm(newQuery);
				passeParTab = false;
			} else {
				boolean allowSearch = true;
				if (etat.equals(ETAT_DETAIL)) {
					UI.addIdToFilter(runtimeData.getParameter("id"));
				}				
				else if (etat.equals(ETAT_TABLEAU_RES)) {
					UI.addIdToFilter("");
				}				
				else {
					allowSearch = UI.updateFilter(runtimeData.getParameters());
				}
				
				if (!allowSearch) {
					xml += "<ALERT>WARN_FORM_NOT_FILLED</ALERT>\n";
					etat = ETAT_FORMULAIRE;
					xml += UI.getForm(newQuery);
				} else {
					xml += UI.getResultXml();
				}
			}
		} else {
			xml += "<ALERT>WARN_CONFIG_NOT_FOUND</ALERT>\n";
			etat = ETAT_FORMULAIRE;
		}
		xml += "</RACINE>\n";
		
		/*traitement de la chaine XML*/
		XSLT xslt = new XSLT(this);
		xslt.setXML(xml);
		
		String etatsuiv = "";
		
		/*Affectation de la feuille XSL en fonction du nombre de rsultat*/
		String feuille = new String();
		if (etat.equals(ETAT_FORMULAIRE)) {
			feuille = "form";
			etatsuiv = ETAT_RECHERCHE;
		} else {
			
			nbRes = UI.getNbReponse();
			
			if (nbRes == 0) {
				feuille = "vide";
				etatsuiv = ETAT_FICHE_RES;
			} else if ((nbRes > 0)&& (nbRes<= conf.getServer(UI.getServerWork()).getDisplay().getMaxCard())) {
				feuille = "fiche";
				etatsuiv = ETAT_FICHE_RES;
			} else if ((nbRes > conf.getServer(UI.getServerWork()).getDisplay().getMaxCard())) {
				feuille = "tabRes";
				etatsuiv = ETAT_TABLEAU_RES;
				passeParTab = true;
			}
			
		}
		
		try {
			xslt.setXSL(xslLocation, feuille, runtimeData.getBrowserInfo());
		} catch (PortalException e) {
			log.error("CAnnuaire::renderXml() : " + e.toString());
		}
		
		
		/*Envoie des paramtres au fichierXSL*/
		xslt.setStylesheetParameter("baseActionURL",runtimeData.getBaseActionURL());
		xslt.setStylesheetParameter("etat", etatsuiv);
		xslt.setStylesheetParameter("annuSelect", UI.getServerWork());
		String rtt = new String();
		if ( (passeParTab) && (etat.equals(ETAT_DETAIL)) )rtt="true"; else rtt="false";
		xslt.setStylesheetParameter("returnToTab", rtt);
		if (UI.getServerWork() != null)xslt.setStylesheetParameter("nomId",conf.getServer(UI.getServerWork()).getQuery().getIdentifiant());
		xslt.setStylesheetParameter("numCanal",staticData.getChannelPublishId());
		if (this.staticData.getParameter("CAnnuaireServantMode") != null) {
			if (this.staticData.getParameter("returnServant").equals("single"))
				xslt.setStylesheetParameter("modeServant", "single");
			else
				xslt.setStylesheetParameter("modeServant", "multiple");
		}
		
		xslt.setTarget(out);
		try {
			xslt.transform();
		} catch (PortalException e) {
			log.error("CAnnuaire::renderXml() : Une erreur est survenue lors du parsing XML/XSL : " + e.toString());
		}
		
	}
	
	/******************************************************************************/
	/** Genration d'une cle pour le cache
	 * 
	 * @return ChannelCacheKey cle du cache
	 */
	public ChannelCacheKey generateKey() {
		if (etat.equals(ETAT_FORMULAIRE)) {
			ChannelCacheKey k = new ChannelCacheKey();
			StringBuffer sbKey = new StringBuffer(1024);
			sbKey.append(this.getClass().getName() + " : ");
			sbKey.append("serverview:").append(UI.getServerView()).append(", ");
			sbKey.append("serverwork:").append(UI.getServerWork()).append(", ");
			sbKey.append("params:").append(runtimeData).append(", ");
			
			k.setKeyScope(ChannelCacheKey.SYSTEM_KEY_SCOPE);
			
			k.setKey(sbKey.toString());
			k.setKeyValidity(new Long(System.currentTimeMillis()));
			return k;
		}
		return null;
	}
	
	/**
	 * put your documentation comment here
	 * @param validity
	 * @return boolean
	 */
	public boolean isCacheValid(Object validity) {
		boolean cacheValid = false;
		if ( etat.equals(ETAT_FORMULAIRE)  ){
			if (validity instanceof Long) {
				Long oldtime = (Long) validity;
				// Si validity  moins de cacheTimeOut secondes le cache est valide
				if (System.currentTimeMillis() - oldtime.longValue()
						< cacheTimeOut * 1000) {
					cacheValid = true;
				}
			}
		}	
		return cacheValid;
	}
	
}
